#!/usr/bin/env python
# coding: utf-8
from cement.core import controller
from nepho.cli import base, scope
import os
import argparse
from termcolor import colored
from textwrap import TextWrapper

from nepho.core import common, cloudlet


# Python 2.x vs 3.x input handling change
try:
    import __builtin__
    input = getattr(__builtin__, 'raw_input')
except (ImportError, AttributeError):
    pass


class NephoCloudletController(base.NephoBaseController):
    class Meta:
        label = 'cloudlet'
        interface = controller.IController
        stacked_on = 'base'
        stacked_type = 'nested'
        description = 'find, download, and manage cloudlets'
        usage = "nepho cloudlet <action> [options]"
        arguments = [
            (['--force', '-f'], dict(action='store_true', dest='force', help=argparse.SUPPRESS)),
            (['--location', '-l'], dict(dest='location', help=argparse.SUPPRESS)),
            (['cloudlet'], dict(help=argparse.SUPPRESS, nargs='?')),
            (['query'], dict(help=argparse.SUPPRESS, nargs='*')),
        ]

    def _setup(self, app):
        super(NephoCloudletController, self)._setup(app)
        self.cloudletManager = cloudlet.CloudletManager(self.app)

    @controller.expose(help="List all installed cloudlets")
    def list(self):
        cloudlets = self.cloudletManager.list()

        dir = ""
        items = list()

        try:
            for cloudlt in cloudlets:  # sorted(all_cloudlets):
                # Print directory if it changes
                cloudlet_path = cloudlt.get_path()
                if dir != os.path.dirname(cloudlet_path):
                    dir = os.path.dirname(cloudlet_path)
                    print colored(dir, "cyan")
                name = os.path.basename(cloudlet_path)

                # If there are multiple versions of a cloudlet with the same name,
                # subsequent versions will be ignored by other commands
                if name not in items:
                    try:
                        y = cloudlt.definition
                    except:
                        print colored(base.DISP_PATH, "yellow"), name, "(", colored("error", "red"), "- missing or malformed cloudlet.yaml )"
                    else:
                        print colored(base.DISP_PATH, "yellow"), name, "(", colored("v%s", "blue") % (y['version']), ")"
                    items.append(name)
                else:
                    print colored(base.DISP_PATH, "yellow"), name, "(", colored("error", "red"), "- duplicate cloudlet will be ignored )"
        except TypeError:
            print colored(base.DISP_PATH, "yellow"), colored("No cloudlets installed.", "blue")

        return

        #cloudlet.list_all_cloudlets(self)

    @controller.expose(help="Describe an installed cloudlet")
    def describe(self):
        if self.app.cloudlet_name is None:
            print "Usage: nepho cloudlet describe <cloudlet>"
            exit(1)
        else:
            scope.print_scope(self)

        c = self.cloudletManager.find(self.app.cloudlet_name)
        if c is None:
            print colored("Error:", "red") + "No cloudlet named \"%s\" found." % (self.cloudlet_name)
            exit(1)
        else:
            wrapper = TextWrapper(width=80, subsequent_indent="              ")
            d = c.definition

            print base.DISP_DASH * 80
            print "Name:         %s" % (d['name'])
            print "Version:      %s" % (d['version'])
            print "Author:       %s" % (d['author'])
            print "License:      %s" % (d['license'])
            print wrapper.fill("Summary:      %s" % (d['summary']))
            print wrapper.fill("Description:  %s" % (d['description']))
            print base.DISP_DASH * 80
        return

    @controller.expose(help="Search the Nepho Cloudlet Registry for cloudlets whose names, summaries, or descriptions match the provided search term")
    def search(self):
        # TODO: Improve search beyond basic string matching

        if self.app.pargs.cloudlet is not None:
            query = self.app.pargs.cloudlet + ' ' + ' '.join(self.app.pargs.query)
        else:
            query = ""

        query = query.strip()

        registry = self.cloudletManager.get_registry()

        if query:
            print "Searching for '%s'" % query + ":"
        else:
            print "Listing all cloudlets:"

        matchList = list()
        for cloudletRepo in registry.keys():
            flattenedText = "%s: %s" % (cloudletRepo, registry[cloudletRepo])
            if (query == '') or (query in flattenedText):
                matchList.append(cloudletRepo)

        if len(matchList) == 0:
            print "No matches found."
        else:
            for cloudletRepo in sorted(matchList):
                cloudletDict = registry[cloudletRepo]

                wrapper = TextWrapper(width=80, subsequent_indent="              ")
                print base.DISP_DASH * 80
                print "Name:         %s" % (cloudletRepo)
                print "Version:      %s" % (cloudletDict['version'])
                print "Author:       %s" % (cloudletDict['author'])
                print "License:      %s" % (cloudletDict['license'])
                print wrapper.fill("Summary:      %s" % (cloudletDict['summary']))

    @controller.expose(help="Install a Nepho cloudlet from the Nepho Cloudlet Registry or from an external Git repository")
    def install(self):
        if self.app.cloudlet_name is None:
            print "Usage: nepho cloudlet install <cloudlet> [--location <location>]"
            exit(1)
        else:
            scope.print_scope(self)

        name = self.app.cloudlet_name
        registry = self.cloudletManager.get_registry()

        if name in registry:
            url = registry[name]['source']
            if self.app.config.get('nepho', 'cloudlet_clone_proto') == "ssh":
                url = url.replace('https://github.com/', 'git@github.com:', 1)
        else:
            if self.app.pargs.location is None:
                print "Cloudlet name was not found in master registry. To install a custom cloudlet, specify a location with the --location option."
                exit(1)
            else:
                url = self.app.pargs.location

        cloudlet_dirs = self.cloudletManager.all_cloudlet_dirs()
        selected_dir = common.select_list(self, cloudlet_dirs, False, "Select an install location:")

        # TODO: Move error handling from core to CLI
        self.cloudletManager.new(name, selected_dir, url)
        return

    @controller.expose(help="Upgrade an installed Nepho cloudlet", aliases=["upgrade"])
    def update(self):
        if self.app.cloudlet_name is None:
            print "Usage: nepho cloudlet update <cloudlet>"
            exit(1)
        else:
            scope.print_scope(self)

        cl = self.cloudletManager.find(self.app.cloudlet_name)
        if cl is None:
            print colored("Error: ", "red") + "Cloudlet is not installed."
            exit(1)

        if not isinstance(cl, list):
            cl = [cl]
        for c in cl:
            try:
                c.update()
            except AssertionError:
                print colored("Error: ", "red") + "Cloudlet update failed, see issue #176."

    @controller.expose(help="Uninstall a Nepho cloudlet", aliases=["remove"])
    def uninstall(self):
        if self.app.cloudlet_name is None:
            print "Usage: nepho cloudlet uninstall [--force/-f] <cloudlet>"
            exit(1)
        else:
            scope.print_scope(self)

        cl = self.cloudletManager.find(self.app.cloudlet_name)
        if cl is None:
            print colored("Error: ", "red") + "Cloudlet is not installed."
            exit(1)

        if not self.app.pargs.force:
            verify = input("Are you sure you want to uninstall %s? [y/N]: " % (self.app.cloudlet_name))
            if verify != 'y' and verify != 'yes':
                exit(1)

        if not isinstance(cl, list):
            cl = [cl]
        for c in cl:
            c.archive(self.app.cloudlet_name, self.app.config.get('nepho', 'archive_dir'))
            c.uninstall()

    @controller.expose(help="Update the local cloudlet registry.", aliases=["update-registry"])
    def registry_update(self):
        self.cloudletManager.clear_registry()
        self.cloudletManager.update_registry()



#!/usr/bin/env python
# coding: utf-8

import argparse
import os
from termcolor import colored
from textwrap import TextWrapper
from pprint import pprint

from cement.core import controller
from nepho.cli import base, scope
from nepho.core import cloudlet


class NephoBlueprintController(base.NephoBaseController):
    class Meta:
        label = 'blueprint'
        interface = controller.IController
        stacked_on = 'base'
        stacked_type = 'nested'
        description = 'list and view individual cloudlet deployment blueprints'
        usage = "nepho blueprint <action> <cloudlet> [blueprint]"
        arguments = [
            (['cloudlet'], dict(help=argparse.SUPPRESS, nargs='?')),
            (['blueprint'], dict(help=argparse.SUPPRESS, nargs='?')),
        ]

    def _setup(self, app):
        super(base.NephoBaseController, self)._setup(app)
        self.cloudletManager = cloudlet.CloudletManager(self.app)

    @controller.expose(help='List all blueprints in a cloudlet')
    def list(self):
        if self.app.cloudlet_name is None:
            print "Usage: nepho blueprint list <cloudlet>"
            exit(1)
        else:
            scope.print_scope(self)

        c = _load_cloudlet(self, self.app.cloudlet_name)
        blueprints = c.blueprints()

        # Prepare to wrap description text
        wrapper = TextWrapper(width=80, initial_indent="        ", subsequent_indent="        ")

        # Now list the available blueprints
        for bp in blueprints:
            if bp.definition is not None:
                print colored("    " + base.DISP_PATH, "yellow"), colored(bp.name, attrs=['underline']), "[", colored(bp.definition['provider'], 'magenta'), "]"
                print wrapper.fill(bp.definition['summary'])
            else:
                print colored("    " + base.DISP_PATH, "yellow"), colored(bp.name, attrs=['underline'])
                print colored("        Error - missing or malformed blueprint.yaml", "red")

        return

    @controller.expose(help='Describe a blueprint')
    def describe(self):
        if self.app.cloudlet_name is None or self.app.blueprint_name is None:
            print "Usage: nepho blueprint describe <cloudlet> <blueprint>"
            exit(1)
        else:
            scope.print_scope(self)

        c = _load_cloudlet(self, self.app.cloudlet_name)
        bp = c.blueprint(self.app.blueprint_name)

        if bp is None:
            print colored("Error: ", "red"), "No blueprint by that name.\nFor a list of blueprints run `nepho blueprint list %s`" % (self.app.cloudlet_name)
            exit(1)

        wrapper  = TextWrapper(width=80, initial_indent="        ", subsequent_indent="        ")
        wrapper2 = TextWrapper(width=80, initial_indent="          ", subsequent_indent="          ")

        if bp.definition is not None:
            print colored("    " + base.DISP_PATH, "yellow"), colored(bp.name, attrs=['underline']), "[", colored(bp.definition['provider'], 'magenta'), "]"
            print wrapper.fill(bp.definition['summary'])
        else:
            print colored("    " + base.DISP_PATH, "yellow"), colored(bp.name, attrs=['underline'])
            print colored("        Error - missing or malformed blueprint.yaml", "red")
            return

        print "\n        Description:"
        print wrapper2.fill(bp.definition['description'])

        print "\n        Default Parameters:"
        params = bp.definition.pop('parameters', None)
        for k, v in params.iteritems():
            print "          %-18s: %s" % (k, v)
        print
        return


def _load_cloudlet(app_obj, name):
    try:
        c = app_obj.cloudletManager.find(name)
    except IOError:
        print colored("Error: ", "red"), "Missing or malformed cloudlet.yaml for %s" % (c.name)
        exit(1)

    if c is None:
        print colored("Error: ", "red"), "Could not find cloudlet %s.\nFor a list of cloudlets run `nepho cloudlet list`" % (name)
        exit(1)

    print colored(os.path.dirname(c.get_path()), "cyan")
    print colored(base.DISP_PATH, "yellow"), c.name, "(", colored("v%s", "blue") % (c.definition['version']), ")"
    return c



import base
import cloudlet
import blueprint
import hooks
import stack
#import config
import parameter
import scope

__all__ = ["base", "hooks", "parameter", "cloudlet", "blueprint", "stack", "scope"]



from cement.core import handler, hook
from cement.utils import test
from nepho import cli
from nepho.cli.base import Nepho
import nose


class NephoTestApp(Nepho):
    class Meta:
        argv = []
        config_files = []


# Test Cloudlet
class a_TestNephoCloudlet(test.CementTestCase):
    app_class = NephoTestApp

    def setUp(self):
        super(a_TestNephoCloudlet, self).setUp()

        self.reset_backend()

        app = self.make_app(argv=['cloudlet', 'registry-update'])
        app.setup()
        app.run()
        app.close()
        app = self.make_app(argv=['cloudlet', '--quiet', 'uninstall', 'nepho-example'])
        app.setup()
        app.run()
        app.close()
        app = self.make_app(argv=['cloudlet', '--quiet', 'install', 'nepho-example'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_cloudlet_registry_update(self):
        app = self.make_app(argv=['cloudlet', 'registry-update'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_cloudlet_update_registry(self):
        app = self.make_app(argv=['cloudlet', 'update-registry'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_cloudlet_list(self):
        app = self.make_app(argv=['cloudlet', 'list'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_cloudlet_search(self):
        app = self.make_app(argv=['cloudlet', 'search', 'nepho-example'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_cloudlet_install(self):
        raise nose.SkipTest('skip this until #164 is implemented')
        app = self.make_app(argv=['cloudlet', 'install', 'nepho-example'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_cloudlet_describe(self):
        app = self.make_app(argv=['cloudlet', 'describe', 'nepho-example'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_cloudlet_update(self):
        app = self.make_app(argv=['cloudlet', 'update', 'nepho-example'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_cloudlet_upgrade(self):
        app = self.make_app(argv=['cloudlet', 'upgrade', 'nepho-example'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_cloudlet_uninstall(self):
        raise nose.SkipTest('skip this until #164 is implemented')
        app = self.make_app(argv=['cloudlet', 'uninstall', 'nepho-example'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_cloudlet_remove(self):
        raise nose.SkipTest('skip this until #164 is implemented')
        app = self.make_app(argv=['cloudlet', 'remove', 'nepho-example'])
        app.setup()
        app.run()
        app.close()



import os
from termcolor import colored
import re


def process_config(app_obj):
    # Multiple cloudlet dirs in a string need to be split into a list and
    # excess whitespace removed
    cloudlet_dirs = app_obj.config.get('nepho', 'cloudlet_dirs').split(',')
    app_obj.cloudlet_dirs = map(lambda x: os.path.expanduser(x.strip()), cloudlet_dirs)
    for dir in [dir for dir in app_obj.cloudlet_dirs if not os.path.exists(dir)]:
        os.makedirs(dir)

    # For all file and directory items, make sure path is expanded
    for key in [key for key in app_obj.config.keys('nepho') if re.search('_(dirs?|file)$', key)]:
        value = app_obj.config.get('nepho', key)
        # Expand user where necessary
        value = os.path.expanduser(value)
        app_obj.config.set('nepho', key, value)

        # If item is a directory, make sure it exists
        if re.search('_dir$', key) and not os.path.exists(value):
            os.makedirs(value)


def set_scope(app_obj):
    """
    Choose and set correct cloudlet and blueprint names to operate on.
    Precedence is configured scope, command line argument, None. If cloudlet is
    configured and only the cloudlet argument is passed, assign it to blueprint.
    """
    if app_obj.config.get('scope', 'cloudlet') is not '':
        app_obj.cloudlet_name = app_obj.config.get('scope', 'cloudlet')
    else:
        try:
            app_obj.cloudlet_name = app_obj.pargs.cloudlet
        except:
            # Nepho is called with no subcommand
            app_obj.cloudlet_name = None

    if app_obj.config.get('scope', 'blueprint') is not '':
        app_obj.blueprint_name = app_obj.config.get('scope', 'blueprint')
    elif hasattr(app_obj.pargs, 'blueprint') and app_obj.pargs.blueprint is not None:
        app_obj.blueprint_name = app_obj.pargs.blueprint
    elif app_obj.config.get('scope', 'cloudlet') is not '':
        app_obj.blueprint_name = app_obj.pargs.cloudlet
    else:
        app_obj.blueprint_name = None



#!/usr/bin/env python
# coding: utf-8
import argparse
import json
import yaml
import collections
from termcolor import colored
from textwrap import TextWrapper, dedent
from pydoc import pager

from cement.core import controller

from nepho.cli import base, scope
from nepho.core import common, cloudlet, stack, provider, provider_factory, scenario, parameter


class NephoStackController(base.NephoBaseController):
    class Meta:
        label = 'stack'
        interface = controller.IController
        stacked_on = 'base'
        stacked_type = 'nested'
        description = 'create, manage, and destroy stacks built from blueprints'
        usage = "nepho stack <action> [options]"
        arguments = [
            (['cloudlet'], dict(help=argparse.SUPPRESS, nargs='?')),
            (['blueprint'], dict(help=argparse.SUPPRESS, nargs='?')),
            (['--save', '-s'], dict(help=argparse.SUPPRESS, action='store_true')),
            (['--name', '-n'], dict(help='custom stack name (aws only)', default=None)),
            (['--develop', '-d'], dict(help='development mode (vagrant only)', action='store_true')),
            (['--params', '-p'], dict(help=argparse.SUPPRESS, nargs='*', action='append')),
        ]

    def _setup(self, app):
        super(base.NephoBaseController, self)._setup(app)
        self.cloudletManager = cloudlet.CloudletManager(self.app)

    @controller.expose(help='Show the context for a stack from a blueprint and configs')
    def show_context(self):
        if self.app.cloudlet_name is None or self.app.blueprint_name is None:
            print "Usage: nepho stack show-context <cloudlet> <blueprint> [-n/--name <stack name] [-s/--save] [-p/--params <param>]"
            exit(1)
        else:
            scope.print_scope(self)

        s = self._assemble_scenario()
        c = s.context

        print "Cloudlet:"
        for k in sorted(c['cloudlet']):
            print "  %-18s: %s" % (k, c['cloudlet'][k])
        print "Blueprint:"
        for k in sorted(c['blueprint']):
            print "  %-18s: %s" % (k, c['blueprint'][k])
        print "Parameters:"
        for k in sorted(c['parameters']):
            print "  %-18s: %s" % (k, c['parameters'][k])

    @controller.expose(help='Validate and display the template output for a stack from a blueprint')
    def validate(self):
        if self.app.cloudlet_name is None or self.app.blueprint_name is None:
            print "Usage: nepho stack validate <cloudlet> <blueprint>"
            exit(1)
        else:
            scope.print_scope(self)

        s = self._assemble_scenario()

        output  = "-" * 80 + "\n"
        output += s.provider.validate_template(s.template)
        output += "\n" + "-" * 80 + "\n"
        output += s.provider.format_template(s.template)
        pager(output)

    @controller.expose(help='Display the raw template for a stack from a blueprint')
    def show_template(self):
        if self.app.cloudlet_name is None or self.app.blueprint_name is None:
            print "Usage: nepho stack show-template <cloudlet> <blueprint>"
            exit(1)
        else:
            scope.print_scope(self)

        s = self._assemble_scenario()

        template_str = self._assemble_scenario().template
        try:
            template_str = s.provider.format_template(template_str)
        except Exception:
            pass

        print template_str

    @controller.expose(help='Create a stack from a blueprint', aliases=['deploy', 'up'])
    def create(self):
        if self.app.cloudlet_name is None or self.app.blueprint_name is None:
            print dedent("""\
                Usage: nepho stack create <cloudlet> <blueprint> [-n/--name <stack name] [-s/--save] [-p/--params <param>]

                -n, --name <stack name>
                  [AWS ONLY] A custom name for your stack rather than nepho-<cloudlet>-<blueprint>.
                  When using this option you must specify name for other stack actions such as
                  connect, destroy, and status or they will not know which stack to act upon.
                -s, --save
                  [NOT IMPLEMENTED] Save command-line (and/or interactive) parameters to an
                  overrides file for use in all future invocations of this command.

                -p, --params
                  Override any parameter from the blueprint template. This option can be passed
                  multiple key=value pairs, and can be called multiple times. If a required
                  parameter is not passed as a command-line option, nepho will interactively
                  prompt for it.

                Examples:
                  nepho stack create my-app development --params AwsAvailZone1=us-east-1a
                  nepho stack create my-app development -s -p Foo=True Bar=False -p Test=Passed""")
            exit(1)
        else:
            scope.print_scope(self)

        s = self._assemble_scenario()
        s.provider.deploy(self.app)

    @controller.expose(help='Check on the status of a stack.')
    def status(self):
        if self.app.cloudlet_name is None or self.app.blueprint_name is None:
            print "Usage: nepho stack status <cloudlet> <blueprint> [-n/--name <stack name]"
            exit(1)
        else:
            scope.print_scope(self)

        s = self._assemble_scenario()

        # For now the provider prints this information
        s.provider.status(self.app)

    @controller.expose(help='Gain access to the stack', aliases=['ssh'])
    def access(self):
        if self.app.cloudlet_name is None or self.app.blueprint_name is None:
            print "Usage: nepho stack access <cloudlet> <blueprint> [-n/--name <stack name]"
            exit(1)
        else:
            scope.print_scope(self)

        s = self._assemble_scenario()

        s.provider.access(self.app)

    @controller.expose(help='Destroy a stack from a blueprint', aliases=['delete', 'down'])
    def destroy(self):
        if self.app.cloudlet_name is None or self.app.blueprint_name is None:
            print "Usage: nepho stack destroy <cloudlet> <blueprint> [-n/--name <stack name]"
            exit(1)
        else:
            scope.print_scope(self)

        s = self._assemble_scenario()
        s.provider.destroy(self.app)

    @controller.expose(help='List running stacks')
    def list(self):
        scope.print_scope(self)

        print "Unimplemented action. (input: %s)" % self.app.pargs.params
        exit(0)

        try:
            cloudlt = self.cloudletManager.find(self.app.cloudlet_name)
            y = cloudlt.definition
        except IOError:
            print colored(base.DISP_PATH, "yellow"), cloudlt.name, "(", colored("error", "red"), "- missing or malformed cloudlet.yaml )"
            exit(1)
        else:
            print colored(base.DISP_PATH, "yellow"), cloudlt.name, "(", colored("v%s", "blue") % (y['version']), ")"

    def _parse_user_params(self):
        """Helper method to extract params from command line into a dict."""
        params = dict()
        if self.app.pargs.params is not None:
            paramList = self.app.pargs.params
            for item in paramList[0]:
                (k, v) = item.split("=")
                params[k] = v
        return params

    def _load_blueprint(self):
        """Helper method to load blueprint & pattern from args."""

        try:
            cloudlt = self.cloudletManager.find(self.app.cloudlet_name)
        except Exception:
            print colored("Error: ", "red"), "Could not load cloudlet %s" % (self.app.cloudlet_name)
            exit(1)

        try:
            bprint = cloudlt.blueprint(self.app.blueprint_name)
        except Exception:
            print colored("Error: ", "red"), "Could not load blueprint %s for cloudlet %s.\nFor a list of blueprints run `nepho blueprint list %s`" % (self.app.blueprint_name, self.app.cloudlet_name, self.app.cloudlet_name)
            exit(1)

        return bprint

    def _assemble_scenario(self):
        """Helper method to create a suitable scenario from the command line options."""

        stored_params = parameter.ParamsManager(self)
        user_params = self._parse_user_params()
        bprint = self._load_blueprint()
        if self.app.pargs.name is not None:
            stack_name = self.app.pargs.name
        else:
            stack_name = '%s-%s' % (self.app.cloudlet_name, self.app.blueprint_name)
        s = scenario.Scenario(bprint, stored_params, user_params, stack_name)

        return s
